@@ -2,11 +2,9 @@ import os, time
 from os.path import *
 import gnomeapplet, gtk, gtk.gdk, gconf, gobject, pango
 from gettext import gettext as _
-import gtk, gobject
-import csv, os
-from gettext import gettext as _
+import locale
+import csv
 import invest, invest.about, invest.chart
-from invest import *
 
 COLORSCALE_POSITIVE = [
 	"white",
@@ -20,7 +18,6 @@ COLORSCALE_POSITIVE = [
 	"#73d216",
 	"#4e9a06",
 ]
-GREEN = COLORSCALE_POSITIVE[-1]
 COLORSCALE_NEGATIVE = [
 	"white",
 	"#fce94f",
@@ -33,7 +30,8 @@ COLORSCALE_NEGATIVE = [
 	"#cc0000",
 	"#a40000",
 ]
-RED = COLORSCALE_NEGATIVE[-1]
+LIGHT = -3
+MEDIUM = -1
 
 TICKER_TIMEOUT = 10000#3*60*1000#
 
@@ -42,32 +40,44 @@ class InvestWidget(gtk.TreeView):
 		gtk.TreeView.__init__(self)
 		self.set_property("rules-hint", True)
 		self.set_reorderable(True)
+		self.set_hover_selection(True)
 
 		simple_quotes_only = quotes_updater.simple_quotes_only()
 
-		# model: SYMBOL, TICKER_ONLY, BALANCE, BALANCE_PCT, VALUE, VARIATION_PCT, PB
+		# model: SYMBOL, LABEL, TICKER_ONLY, BALANCE, BALANCE_PCT, VALUE, VARIATION_PCT, PB
 		# Translators: these words all refer to a stock. Last is short
-		# for "last price". Gain is referring to the gain since the 
+		# for "last price". Gain is referring to the gain since the
 		# stock was purchased.
 		col_names = [_('Ticker'), _('Last'), _('Change %'), _('Chart'), _('Gain'), _('Gain %')]
-		col_cellgetdata_functions = [self._getcelldata_symbol, self._getcelldata_value,
+		col_cellgetdata_functions = [self._getcelldata_label, self._getcelldata_value,
 			self._getcelldata_variation, None, self._getcelldata_balance, 
 			self._getcelldata_balancepct]
 		for i, col_name in enumerate(col_names):
 			if i < 3:
 				cell = gtk.CellRendererText()
+				if i > 0:
+					cell.set_property("xalign", 1.0)
 				column = gtk.TreeViewColumn (col_name, cell)
+				if i == 0:
+					column.set_sort_column_id(quotes_updater.LABEL)
+				elif i == 2:
+					column.set_sort_column_id(quotes_updater.VARIATION_PCT)
 				column.set_cell_data_func(cell, col_cellgetdata_functions[i])
 				self.append_column(column)
 			elif i == 3:
 				cell_pb = gtk.CellRendererPixbuf()
-				column = gtk.TreeViewColumn (col_name, cell_pb, pixbuf=6)
+				column = gtk.TreeViewColumn (col_name, cell_pb, pixbuf=quotes_updater.PB)
 				self.append_column(column)
 			else:
 				# add the last two column only if we have any positions
 				if simple_quotes_only == False:
 					cell = gtk.CellRendererText()
+					cell.set_property("xalign", 1.0)
 					column = gtk.TreeViewColumn (col_name, cell)
+					if i == 4:
+						column.set_sort_column_id(quotes_updater.BALANCE)
+					elif i == 5:
+						column.set_sort_column_id(quotes_updater.BALANCE_PCT)
 					column.set_cell_data_func(cell, col_cellgetdata_functions[i])
 					self.append_column(column)
 
@@ -76,42 +86,68 @@ class InvestWidget(gtk.TreeView):
 
 		self.connect('row-activated', self.on_row_activated)
 		self.set_model(quotes_updater)
-		
-	def _getcelldata_symbol(self, column, cell, model, iter):
-		cell.set_property('text', model[iter][model.SYMBOL])
+
+
+	# locale-aware formatting of the value as monetary, without currency symbol, using 2 decimal digits
+	def format_currency(self, value, currency):
+		return locale.format("%.2f", value, True, True) + " " + currency
+
+	# locale-aware formatting of the percent float (decimal point, thousand grouping point) with 2 decimal digits
+	def format_percent(self, value):
+		return locale.format("%+.2f", value, True) + "%"
+
+	# locale-aware formatting of the float value (decimal point, thousand grouping point) with sign and 2 decimal digits
+	def format_difference(self, value, currency):
+		return locale.format("%+.2f", value, True, True) + " " + currency
+
+
+	def _getcelldata_label(self, column, cell, model, iter):
+		cell.set_property('text', model[iter][model.LABEL])
 
 	def _getcelldata_value(self, column, cell, model, iter):
-		cell.set_property('text', "%.5g" % model[iter][model.VALUE])
+		cell.set_property('text', self.format_currency(model[iter][model.VALUE], model[iter][model.CURRENCY]))
+
+	def is_selected(self, model, iter):
+		m, it = self.get_selection().get_selected()
+		return it != None and model.get_path(iter) == m.get_path(it)
+
+	def get_color(self, model, iter, field):
+		palette = COLORSCALE_POSITIVE
+		intensity = MEDIUM
+		if model[iter][field] < 0:
+			palette = COLORSCALE_NEGATIVE
+		if self.is_selected(model, iter):
+			intensity = LIGHT
+		return palette[intensity]
 
 	def _getcelldata_variation(self, column, cell, model, iter):
-		color = GREEN
-		if model[iter][model.VARIATION_PCT] < 0: color = RED
-		change_pct = model[iter][model.VARIATION_PCT]
+		color = self.get_color(model, iter, model.VARIATION_PCT)
+		change_pct = self.format_percent(model[iter][model.VARIATION_PCT])
 		cell.set_property('markup',
-			"<span foreground='%s'>%+.2f%%</span>" %
+			"<span foreground='%s'>%s</span>" %
 			(color, change_pct))
 
 	def _getcelldata_balance(self, column, cell, model, iter):
 		is_ticker_only = model[iter][model.TICKER_ONLY]
-		color = GREEN
-		if model[iter][model.BALANCE] < 0: color = RED
+		color = self.get_color(model, iter, model.BALANCE)
 		if is_ticker_only:
 			cell.set_property('text', '')
 		else:
-					cell.set_property('markup',
-			"<span foreground='%s'>%+.2f</span>" %
-			(color, model[iter][model.BALANCE]))
+			balance = self.format_difference(model[iter][model.BALANCE], model[iter][model.CURRENCY])
+			cell.set_property('markup',
+				"<span foreground='%s'>%s</span>" %
+				(color, balance))
 
 	def _getcelldata_balancepct(self, column, cell, model, iter):
 		is_ticker_only = model[iter][model.TICKER_ONLY]
-		color = GREEN
-		if model[iter][model.BALANCE] < 0: color = RED
+		color = self.get_color(model, iter, model.BALANCE_PCT)
 		if is_ticker_only:
 			cell.set_property('text', '')
 		else:
-					cell.set_property('markup',
-			"<span foreground='%s'>%+.2f%%</span>" %
-			(color, model[iter][model.BALANCE_PCT]))
+			balance_pct = self.format_percent(model[iter][model.BALANCE_PCT])
+			cell.set_property('markup',
+				"<span foreground='%s'>%s</span>" %
+				(color, balance_pct))
 
 	def on_row_activated(self, treeview, path, view_column):
 		ticker = self.get_model()[self.get_model().get_iter(path)][0]
@@ -123,45 +159,45 @@ class InvestWidget(gtk.TreeView):
 #class InvestTicker(gtk.Label):
 #	def __init__(self):
 #		gtk.Label.__init__(self, _("Waiting..."))
-#		
+#
 #		self.quotes = []
 #		gobject.timeout_add(TICKER_TIMEOUT, self.scroll_quotes)
-#		
+#
 #		get_quotes_updater().connect('quotes-updated', self.on_quotes_update)
-#						
+#
 #	def on_quotes_update(self, updater):
 #		self.quotes = []
 #		updater.foreach(self.update_quote, None)
-#	
+#
 #	def update_quote(self, model, path, iter, user_data):
 #		color = GREEN
 #		if model[iter][model.BALANCE] < 0:
 #			color = RED
-#		
+#
 #		self.quotes.append(
 #			"%s: <span foreground='%s'>%+.2f (%+.2f%%)</span> %.2f" %
 #			(model[iter][model.SYMBOL], color, model[iter][model.BALANCE], model[iter][model.BALANCE_PCT], model[iter][model.VALUE]))
-#				
+#
 #	def scroll_quotes(self):
 #		if len(self.quotes) == 0:
 #			return True
-#		
+#
 #		q = self.quotes.pop()
 #		self.set_markup("<span face='Monospace'>%s</span>" % q)
 #		self.quotes.insert(0, q)
-#		
+#
 #		return True
 #
 #gobject.type_register(InvestTicker)
 
-class InvestTrend(gtk.Image):	
+class InvestTrend(gtk.Image):
 	def __init__(self):
 		gtk.Image.__init__(self)
 		self.pixbuf = None
 		self.previous_allocation = (0,0)
 		self.connect('size-allocate', self.on_size_allocate)
 		get_quotes_updater().connect('quotes-updated', self.on_quotes_update)
-	
+
 	def on_size_allocate(self, widget, allocation):
 		if self.previous_allocation == (allocation.width, allocation.height):
 			return
@@ -169,7 +205,7 @@ class InvestTrend(gtk.Image):
 		self.pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, allocation.height, allocation.height)
 		self.set_color("grey")
 		self.previous_allocation = (allocation.width, allocation.height)
-		
+
 	def set_color(self, color, opacity=0xFF):
 		if self.pixbuf != None:
 			try:
@@ -178,8 +214,8 @@ class InvestTrend(gtk.Image):
 				self.pixbuf.fill(
 					int(color.red*factor)<<24|int(color.green*factor)<<16|int(color.blue*factor)<<8|opacity)
 				self.set_from_pixbuf(self.pixbuf)
-			except Exception, e:
-				print e
+			except Exception, msg:
+				invest.error("Could not set color: %s" % msg)
 
 	def on_quotes_update(self, updater):
 		start_total = 0
@@ -188,16 +224,16 @@ class InvestTrend(gtk.Image):
 			# Don't count the ticker only symbols in the color-trend
 			if row[updater.TICKER_ONLY]:
 				continue
-				
+
 			var = row[updater.VARIATION]/100
 			now = row[updater.VALUE]
 
 			start = now / (1 + var)
-			
-			portfolio_number = sum([purchase["amount"] for purchase in invest.STOCKS[row[updater.SYMBOL]]])
+
+			portfolio_number = sum([purchase["amount"] for purchase in invest.STOCKS[row[updater.SYMBOL]]["purchases"]])
 			start_total += start * portfolio_number
 			now_total += now * portfolio_number
-		
+
 		day_var = 0
 		if start_total != 0:
 			day_var = (now_total - start_total) / start_total * 100
@@ -210,5 +246,5 @@ class InvestTrend(gtk.Image):
 			color = COLORSCALE_POSITIVE[min(len(COLORSCALE_POSITIVE)-1, abs(color))]
 
 		self.set_color(color, opacity)
-	
+
 gobject.type_register(InvestTrend)
